package com.frostbite.wf;

import java.io.Serializable;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.Singleton;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.Session;

import org.apache.activemq.artemis.api.core.TransportConfiguration;
import org.apache.activemq.artemis.api.jms.ActiveMQJMSClient;
import org.apache.activemq.artemis.api.jms.JMSFactoryType;
import org.apache.activemq.artemis.core.remoting.impl.invm.InVMConnectorFactory;

@Singleton
public class JmsPublisher {
    /** connection. */
    private Connection connection;
    /** producer. */
    private MessageProducer producer;
    /** session. */
    private Session session;

    /**
     * start queue.
     */
    @PostConstruct
    private void startup() {
        System.out.println("Starting up");
        final TransportConfiguration transportConfiguration = new TransportConfiguration(InVMConnectorFactory.class.getName());
        final ConnectionFactory cf = ActiveMQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, transportConfiguration);
        final Queue queue = ActiveMQJMSClient.createQueue("java:/jms/queue/Avi");
        try {
            connection = cf.createConnection();
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            producer = session.createProducer(queue);
            connection.start();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * shutdown queue.
     */
    @PreDestroy
    private void shutdown() {
        System.out.println("going down*****");
        if (session != null) {
            try {
                session.close();
                System.out.println("session successfuly closed");
            } catch (final Exception e) {
                System.out.println("failed to close session: " + e.getMessage());
                e.printStackTrace();
            }
        }
        if (connection != null) {
            try {
                connection.close();
                System.out.println("connection successfuly closed");
            } catch (final JMSException e) {
                System.out.println("failed to close connection: " + e.getMessage());
                e.printStackTrace();
            }
        }
    }

    /**
     * send message.
     * @param msg message
     */
    public void publish(final Serializable msg) {
        try {
            // 2. Sending ObjectMessage to the Queue
            final ObjectMessage objMsg = session.createObjectMessage(msg);
            producer.send(objMsg);
            System.out.println("-2. Sent ObjectMessage to the Queue");
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

}
